@@ -3,10 +3,8 @@
  * @brief File containing the main function and argument parsing.
  * @parblock
  * SortMeRNA - next-generation reads filter for metatranscriptomic or total RNA
- * @copyright Copyright (C) 2012-2014 Bonsai Bioinformatics Research Group, LIFL and 
- * INRIA Nord-Europe, France
- * OTU-picking extensions developed in the Knight Lab, BioFrontiers Institute,
- * University of Colorado at Boulder, Boulder, CO
+ * @copyright 2013-15 Bonsai Bioinformatics Research Group
+ * 2014-15 Knight Lab, Department of Pediatrics, UCSD, La Jolla
  *
  * SortMeRNA is free software: you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -57,7 +55,8 @@ long unsigned int map_size_gv = pagesize_gv;
 bool map_size_set_gv = false;
 bool samout_gv = false;
 bool blastout_gv = false;
-int32_t blast_outfmt = -1;
+vector<string> user_opts;
+bool blast_tabular = false;
 bool fastxout_gv = false;
 bool otumapout_gv = false;
 int32_t min_lis_gv = -1;
@@ -66,10 +65,10 @@ int32_t seed_hits_gv = -1;
 int32_t edges_gv = -1;
 bool full_search_gv = false;
 /*! @brief Version number */
-char version_num[] = "2.0, 29/11/2014";
+char version_num[] = "2.1, 01/02/2016";
 bool as_percent_gv = false;
 bool pid_gv = false;
-int16_t num_best_hits_gv = 0;
+int32_t num_best_hits_gv = 0;
 
 
 
@@ -81,10 +80,9 @@ int16_t num_best_hits_gv = 0;
 void welcome()
 {
   printf("\n  Program:     SortMeRNA version %s\n",version_num );
-  printf("  Copyright:   2012-2015 Bonsai Bioinformatics Research Group:\n");
+  printf("  Copyright:   2012-16 Bonsai Bioinformatics Research Group:\n");
   printf("               LIFL, University Lille 1, CNRS UMR 8022, INRIA Nord-Europe\n" );
-  printf("               OTU-picking extensions and continuing support developed in the Knight Lab,\n");
-  printf("               BioFrontiers Institute, University of Colorado at Boulder\n");
+  printf("               2014-16 Knight Lab, Department of Pediatrics, UCSD, La Jolla,\n");
   printf("  Disclaimer:  SortMeRNA comes with ABSOLUTELY NO WARRANTY; without even the\n");
   printf("               implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n");
   printf("               See the GNU Lesser General Public License for more details.\n");
@@ -122,11 +120,14 @@ void printlist()
   printf("     %s--sam%s             %sBOOL%s            output SAM alignment                                           %soff%s\n","\033[1m","\033[0m","\033[4m","\033[0m","\033[4m","\033[0m");
   printf("                                         (for aligned reads only)\n");
   printf("     %s--SQ%s              %sBOOL%s            add SQ tags to the SAM file                                    %soff%s\n","\033[1m","\033[0m","\033[4m","\033[0m","\033[4m","\033[0m");
-  printf("     %s--blast%s           %sINT%s             output alignments in various Blast-like formats                \n","\033[1m","\033[0m","\033[4m","\033[0m");
-  printf("                                        0 - pairwise\n");
-  printf("                                        1 - tabular (Blast -m 8 format)\n");
-  printf("                                        2 - tabular + column for CIGAR \n");
-  printf("                                        3 - tabular + columns for CIGAR and query coverage\n");
+  printf("     %s--blast%s           %sSTRING%s          output alignments in various Blast-like formats                \n","\033[1m","\033[0m","\033[4m","\033[0m");
+  printf("                                        '0' - pairwise\n");
+  printf("                                        '1' - tabular (Blast -m 8 format)\n");
+  printf("                                        '1 cigar' - tabular + column for CIGAR \n");
+  printf("                                        '1 cigar qcov' - tabular + columns for CIGAR\n");
+  printf("                                                         and query coverage\n");
+  printf("                                        '1 cigar qcov qstrand' - tabular + columns for CIGAR,\n");
+  printf("                                                                query coverage and strand\n");
   printf("     %s--log%s             %sBOOL%s            output overall statistics                                      %soff%s\n","\033[1m","\033[0m","\033[4m","\033[0m","\033[4m","\033[0m");
 #ifdef NOMASK_option
   printf("     %s--no-mask%s         %sBOOL%s            do not mask low occurrence (L/2)-mers when searching           %son%s\n","\033[1m","\033[0m","\033[4m","\033[0m","\033[4m","\033[0m");
@@ -800,19 +801,72 @@ main(int argc,
         {
           if ( blastout_gv )
           {
-            fprintf(stderr,"\n  %sERROR%s: --blast [INT] has already been set once.\n\n",
+            fprintf(stderr,"\n  %sERROR%s: --blast [STRING] has already been set once.\n\n",
                     "\033[0;31m","\033[0m");
             exit(EXIT_FAILURE);
           }
           else
           {
-            if ( (sscanf(argv[narg+1],"%d",&blast_outfmt) != 1) || 
-                 (blast_outfmt < 0) || (blast_outfmt > 3))
+            string str(argv[narg+1]);
+            // split blast options into vector by space
+            istringstream iss(str);
+            do
             {
-              fprintf(stderr,"\n  %sERROR%s: --blast [INT] must be a positive integer "
-                      "with value 0<=INT<=3.\n\n","\033[0;31m","\033[0m");
+              string s;
+              iss >> s;
+              user_opts.push_back(s);
+            } while (iss);
+
+            // remove the end of file entry
+            user_opts.pop_back();
+
+            bool blast_human_readable = false;
+            vector<string> supported_opts;
+            supported_opts.push_back("0");
+            supported_opts.push_back("1");
+            supported_opts.push_back("cigar");
+            supported_opts.push_back("qstrand");
+            supported_opts.push_back("qcov");
+
+            // check user options are supported
+            for ( uint32_t i = 0; i < user_opts.size(); i++ )
+            {
+              bool match_found = false;
+              string opt = user_opts[i];
+              vector<string>::iterator it;
+              for ( it = supported_opts.begin(); it != supported_opts.end(); ++it )
+              {
+                if ( opt.compare(*it) == 0 )
+                {
+                  if ( opt.compare("0") == 0 ) blast_human_readable = true;
+                  else if ( opt.compare("1") == 0 ) blast_tabular = true;
+                  match_found = true;
+                  break;
+                }
+              }
+              if ( !match_found )
+              {
+                fprintf(stderr,"\n  %sERROR%s: `%s` is not supported in --blast [STRING].\n\n",
+                               "\033[0;31m","\033[0m", opt.c_str());
+                exit(EXIT_FAILURE);               
+              }
+            }
+
+            // more than 1 field with blast human-readable format given
+            if ( blast_human_readable && (user_opts.size() > 1 ) )
+            {
+              fprintf(stderr,"\n  %sERROR%s: for human-readable format, --blast [STRING] cannot contain "
+                             "more fields than '0'.\n\n", "\033[0;31m","\033[0m");
               exit(EXIT_FAILURE);
             }
+            // both human-readable and tabular format options have been chosen
+            if ( blast_human_readable && blast_tabular )
+            {
+              fprintf(stderr,"\n  %sERROR%s: --blast [STRING] can only have one of the options "
+                             "'0' (human-readable) or '1' (tabular).\n\n", "\033[0;31m","\033[0m");
+              exit(EXIT_FAILURE);
+            }
+
             blastout_gv = true;
             narg+=2;
           }
@@ -865,10 +919,9 @@ main(int argc,
           }
           else
           {
-            if ( (sscanf(argv[narg+1],"%hd",&num_best_hits_gv) != 1) ||
-                 (num_best_hits_gv <= 0) )
+            if ( (sscanf(argv[narg+1],"%d",&num_best_hits_gv) != 1 ) )
             {
-              fprintf(stderr,"\n  %sERROR%s: --best [INT] must be > 0.\n\n",
+              fprintf(stderr,"\n  %sERROR%s: could not read --best [INT] as integer.\n\n",
                       "\033[0;31m","\033[0m");
               exit(EXIT_FAILURE);
             }
@@ -1301,10 +1354,10 @@ main(int argc,
   }
     
   // If --num_alignments output was chosen, check an alignment format has also been chosen
-  if ( num_alignments_gv_set && !(blastout_gv || samout_gv) )
+  if ( num_alignments_gv_set && !(blastout_gv || samout_gv || fastxout_gv) )
   {
     fprintf(stderr,"\n  %sERROR%s: --num_alignments [INT] has been set but no output "
-            "format has been chosen (--blast or --sam).\n\n","\033[0;31m","\033[0m");
+            "format has been chosen (--blast, --sam or --fastx).\n\n","\033[0;31m","\033[0m");
     exit(EXIT_FAILURE);
   }
 
@@ -1324,11 +1377,12 @@ main(int argc,
     exit(EXIT_FAILURE);
   }
         
-  // Option --print_all_reads can only be used with Blast-like tabular formats (not pairwise)
-  if ( print_all_reads_gv && blastout_gv && (blast_outfmt < 1) )
+  // Option --print_all_reads can only be used with Blast-like tabular
+  // and SAM formats (not pairwise)
+  if ( print_all_reads_gv && blastout_gv && !blast_tabular )
   {
-    fprintf(stderr,"\n  %sERROR%s: --print_all_reads [BOOL] can only be used for BLAST "
-            "output formats 1,2 and 3 (using --blast INT).\n\n","\033[0;31m","\033[0m");
+    fprintf(stderr,"\n  %sERROR%s: --print_all_reads [BOOL] can only be used with BLAST-like "
+            "tabular format.\n\n","\033[0;31m","\033[0m");
     exit(EXIT_FAILURE);
   }
 
@@ -1367,7 +1421,7 @@ main(int argc,
     fprintf(stderr,"\n  %sERROR%s: --id [INT] and --coverage [INT] can only be used "
             "together with --otu_map.\n","\033[0;31m","\033[0m");
     fprintf(stderr,"  These two options are used for constructing the OTU map by "
-            "filtering the alignments passing the E-value threshold.\n\n");
+            "filtering alignments passing the E-value threshold.\n\n");
     exit(EXIT_FAILURE);
   }
       
